@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004 Damien Miller <djm@mindrot.org>
+ * Copyright (c) 2004,2005 Damien Miller <djm@mindrot.org>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,8 +14,9 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#include <sys/types.h>
+#include "flowd-common.h"
 
+#include <sys/types.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -26,7 +27,7 @@
 #include "filter.h"
 #include "store.h"
 
-RCSID("$Id: filter.c,v 1.18 2005/05/14 06:04:48 djm Exp $");
+RCSID("$Id: filter.c,v 1.25 2005/12/12 03:33:51 djm Exp $");
 
 /* #define FILTER_DEBUG */
 
@@ -148,19 +149,19 @@ format_rule(const struct filter_rule *rule)
 		strlcat(rulebuf, tmpbuf, sizeof(rulebuf));
 	}
 	if (rule->match.match_what & FF_MATCH_DAYTIME &&
-	    rule->match.after != -1) {
+	    rule->match.dayafter != -1) {
 		snprintf(tmpbuf, sizeof(tmpbuf), "after %02d:%02d:%02d ",
-		    rule->match.after / 3600, 
-		    (rule->match.after / 60) % 60, 
-		    rule->match.after % 60);
+		    rule->match.dayafter / 3600, 
+		    (rule->match.dayafter / 60) % 60, 
+		    rule->match.dayafter % 60);
 		strlcat(rulebuf, tmpbuf, sizeof(rulebuf));
 	}
 	if (rule->match.match_what & FF_MATCH_DAYTIME &&
-	    rule->match.before != -1) {
+	    rule->match.daybefore != -1) {
 		snprintf(tmpbuf, sizeof(tmpbuf), "before %02d:%02d:%02d ",
-		    rule->match.before / 3600, 
-		    (rule->match.before / 60) % 60, 
-		    rule->match.before % 60);
+		    rule->match.daybefore / 3600, 
+		    (rule->match.daybefore / 60) % 60, 
+		    rule->match.daybefore % 60);
 		strlcat(rulebuf, tmpbuf, sizeof(rulebuf));
 	}
 
@@ -175,21 +176,21 @@ format_rule(const struct filter_rule *rule)
 }
 
 static int
-flow_time_match(time_t recv_secs, int day_mask, int after, int before)
+flow_daytime_match(time_t recv_sec, int day_mask, int dayafter, int daybefore)
 {
 	struct tm *tm;
-	int secs;
+	int sec;
 
-	tm = localtime(&recv_secs);
+	tm = localtime(&recv_sec);
 
 	if (day_mask != 0 && (day_mask & (1 << tm->tm_wday)) == 0)
 		return (0);
 
-	secs = tm->tm_sec + (tm->tm_min * 60) + (tm->tm_hour * 3600);
+	sec = tm->tm_sec + (tm->tm_min * 60) + (tm->tm_hour * 3600);
 
-	if ((before != -1) && secs > before)
+	if ((daybefore != -1) && sec > daybefore)
 		return (0);
-	if ((after != -1) && secs < after)
+	if ((dayafter != -1) && sec < dayafter)
 		return (0);
 	
 	return (1);
@@ -200,6 +201,7 @@ flow_match(const struct filter_rule *rule,
     const struct store_flow_complete *flow)
 {
 	int m;
+	u_int tt;
 
 #define FRNEG(what) (rule->match.match_negate & FF_MATCH_##what)
 #define FRMATCH(what) (rule->match.match_what & FF_MATCH_##what)
@@ -214,10 +216,8 @@ flow_match(const struct filter_rule *rule,
 	}
 
 	if (FRMATCH(AF)) {
-		m = ((FRMATCH(SRC_ADDR) &&
-		    rule->match.src_addr.af == rule->match.af) ||
-		    (FRMATCH(DST_ADDR) &&
-		    rule->match.dst_addr.af == rule->match.af));
+		m = flow->src_addr.af == rule->match.af ||
+		    flow->dst_addr.af == rule->match.af;
 		FRRET(AF);
 	}
 
@@ -258,12 +258,22 @@ flow_match(const struct filter_rule *rule,
 		    rule->match.tcp_flags_equals);
 		FRRET(TCP_FLAGS);
 	}
+
+	tt = ntohl(flow->recv_time.recv_sec);
+
 	if (FRMATCH(DAYTIME)) {
-		m = flow_time_match(ntohl(flow->recv_time.recv_secs), 
-		    rule->match.day_mask, rule->match.after,
-		    rule->match.before);
+		m = flow_daytime_match(tt, rule->match.day_mask,
+		    rule->match.dayafter, rule->match.daybefore);
 		FRRET(DAYTIME);
 	}
+	if (FRMATCH(ABSTIME)) {
+		m = 1;
+		if (rule->match.absbefore > 0)
+			m &= tt < rule->match.absbefore;
+		if (rule->match.absafter > 0)
+			m &= tt > rule->match.absafter;
+		FRRET(ABSTIME);
+	}
 
 #undef FRMATCH
 #undef FRNEG
