@@ -1,7 +1,7 @@
-/*	$Id: filter.h,v 1.13 2005/05/14 06:04:48 djm Exp $	*/
+/*	$Id: filter.h,v 1.17 2005/12/12 03:33:51 djm Exp $	*/
 
 /*
- * Copyright (c) 2004 Damien Miller <djm@mindrot.org>
+ * Copyright (c) 2004,2005 Damien Miller <djm@mindrot.org>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -21,7 +21,7 @@
 #ifndef _FILTER_H
 #define _FILTER_H
 
-#include "common.h"
+#include "flowd-common.h"
 #include "sys-queue.h"
 #include "addr.h"
 #include "store.h"
@@ -44,6 +44,7 @@ struct filter_action {
 #define FF_MATCH_TCP_FLAGS	(1<<7)
 #define FF_MATCH_AF		(1<<8)
 #define FF_MATCH_DAYTIME	(1<<9)
+#define FF_MATCH_ABSTIME	(1<<10)
 struct filter_match {
 	u_int32_t	match_what;
 	u_int32_t	match_negate;
@@ -61,8 +62,10 @@ struct filter_match {
 	int		tcp_flags_mask;
 	int		tcp_flags_equals;
 	int		day_mask;
-	int		after;
-	int		before;
+	int		dayafter;
+	int		daybefore;
+	int		absafter;
+	int		absbefore;
 };
 
 struct filter_rule {
