@@ -1,4 +1,4 @@
-/*	$Id: flowd.h,v 1.16 2005/04/28 09:02:58 djm Exp $	*/
+/*	$Id: flowd.h,v 1.18 2005/10/13 11:27:44 djm Exp $	*/
 
 /*
  * Copyright (c) 2004,2005 Damien Miller <djm@mindrot.org>
@@ -24,7 +24,7 @@
 #include <stdarg.h>
 #include <syslog.h>
 
-#include "common.h"
+#include "flowd-common.h"
 #include "sys-queue.h"
 #include "addr.h"
 #include "filter.h"
@@ -71,6 +71,7 @@ TAILQ_HEAD(join_groups, join_group);
 #define FLOWD_OPT_INSECURE		(1<<2)
 struct flowd_config {
 	char			*log_file;
+	char			*log_socket;
 	char			*pid_file;
 	u_int32_t		store_mask;
 	u_int32_t		opts;
