@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 
-# Copyright (c) 2004 Damien Miller <djm@mindrot.org>
+# Copyright (c) 2004,2005 Damien Miller <djm@mindrot.org>
 #
 # Permission to use, copy, modify, and distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -14,7 +14,7 @@
 # ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 # OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
-# $Id: reader.pl,v 1.8 2004/10/31 06:21:49 djm Exp $
+# $Id: reader.pl,v 1.10 2005/08/21 11:16:05 djm Exp $
 
 # This intended to be an example of the Flowd package API more than a usable
 # application
@@ -35,12 +35,10 @@ usage() unless (defined $ARGV[0]);
 foreach my $ffile (@ARGV) {
 	my $log = Flowd->new($ffile);
 	
-	printf "LOGFILE %s started at %s\n",
-	    $ffile, Flowd::iso_time($log->{start_time}, 0);
+	printf "LOGFILE %s \n", $ffile;
 	
 	while (my $flow = $log->read_flow()) {
-		print $log->format(Flowd::BRIEF, 0, $flow);
-		print "\n";
+		print $log->format(Flowd::BRIEF, 0, $flow) . "\n";
 	}
 	$log->finish();
 }
